From b99f5232c33aba9877274a95df04fdf2a9e1e6a3 Mon Sep 17 00:00:00 2001
From: Kaloyan Kovachev <kkovachev@varna.net>
Date: Wed, 4 Jun 2008 22:18:15 +0300
Subject: [PATCH] Move the CSA Key parsing inside csa_SetCW function

---
 modules/demux/ts.c     |   36 ++++++--------------------------
 modules/mux/mpeg/csa.c |   52 ++++++++++++++++++++++++++++++++++++++++++-----
 modules/mux/mpeg/csa.h |    2 +-
 modules/mux/mpeg/ts.c  |   31 +++++-----------------------
 4 files changed, 60 insertions(+), 61 deletions(-)

diff --git a/modules/demux/ts.c b/modules/demux/ts.c
index 68a61c5..3e24874 100644
--- a/modules/demux/ts.c
+++ b/modules/demux/ts.c
@@ -777,41 +777,20 @@ static int Open( vlc_object_t *p_this )
     var_Get( p_demux, "ts-csa-ck", &val );
     if( val.psz_string && *val.psz_string )
     {
-        char *psz = val.psz_string;
-        if( psz[0] == '0' && ( psz[1] == 'x' || psz[1] == 'X' ) )
-        {
-            psz += 2;
-        }
-        if( strlen( psz ) != 16 )
+        int i_res;
+
+        p_sys->csa = csa_New();
+
+        i_res = csa_SetCW( (vlc_object_t*)p_demux, p_sys->csa, val.psz_string, 1 );
+        if( i_res != VLC_SUCCESS || csa_SetCW( (vlc_object_t*)p_demux, p_sys->csa, val.psz_string, 0 ) != VLC_SUCCESS )
         {
-            msg_Warn( p_demux, "invalid csa ck (it must be 16 chars long)" );
+            csa_Delete( p_sys->csa );
         }
-        else
-        {
-#ifndef UNDER_CE
-            uint64_t i_ck = strtoull( psz, NULL, 16 );
-#else
-            uint64_t i_ck = strtoll( psz, NULL, 16 );
-#endif
-            uint8_t ck[8];
-            int     i;
-            for( i = 0; i < 8; i++ )
-            {
-                ck[i] = ( i_ck >> ( 56 - 8*i) )&0xff;
-            }
-#ifndef TS_NO_CSA_CK_MSG
-            msg_Dbg( p_demux, "using CSA scrambling with "
-                     "ck=%x:%x:%x:%x:%x:%x:%x:%x",
-                     ck[0], ck[1], ck[2], ck[3], ck[4], ck[5], ck[6], ck[7] );
-#endif
-            p_sys->csa = csa_New();
 
             if( p_sys->csa )
             {
                 vlc_value_t pkt_val;
 
-                csa_SetCW( p_sys->csa, ck, ck );
-
                 var_Create( p_demux, "ts-csa-pkt", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
                 var_Get( p_demux, "ts-csa-pkt", &pkt_val );
                 if( pkt_val.i_int < 4 || pkt_val.i_int > 188 )
@@ -823,7 +802,6 @@ static int Open( vlc_object_t *p_this )
                 else p_sys->i_csa_pkt_size = pkt_val.i_int;
                 msg_Dbg( p_demux, "decrypting %d bytes of packet", p_sys->i_csa_pkt_size );
             }
-        }
     }
     free( val.psz_string );
 
diff --git a/modules/mux/mpeg/csa.c b/modules/mux/mpeg/csa.c
index 54545bf..94f7f14 100644
--- a/modules/mux/mpeg/csa.c
+++ b/modules/mux/mpeg/csa.c
@@ -81,13 +81,53 @@ void   csa_Delete( csa_t *c )
 /*****************************************************************************
  * csa_SetCW:
  *****************************************************************************/
-void csa_SetCW( csa_t *c, uint8_t o_ck[8], uint8_t e_ck[8] )
+int csa_SetCW( vlc_object_t *p_caller, csa_t *c, char *psz_ck, int set_odd )
 {
-    memcpy( c->o_ck, o_ck, 8 );
-    csa_ComputeKey( c->o_kk, o_ck );
+    if ( !c )
+    {
+        msg_Dbg( p_caller, "no CSA found" );
+        return VLC_EGENERIC;
+    }
+    /* skip 0x */
+    if( psz_ck[0] == '0' && ( psz_ck[1] == 'x' || psz_ck[1] == 'X' ) )
+    {
+        psz_ck += 2;
+    }
+    if( strlen( psz_ck ) != 16 )
+    {
+        msg_Warn( p_caller, "invalid csa ck (it must be 16 chars long)" );
+        return VLC_EGENERIC;
+    }
+    else
+    {
+#ifndef UNDER_CE
+        uint64_t i_ck = strtoull( psz_ck, NULL, 16 );
+#else
+        uint64_t i_ck = strtoll( psz_ck, NULL, 16 );
+#endif
+        uint8_t  ck[8];
+        int      i;
 
-    memcpy( c->e_ck, e_ck, 8 );
-    csa_ComputeKey( c->e_kk, e_ck );
+        for( i = 0; i < 8; i++ )
+        {
+            ck[i] = ( i_ck >> ( 56 - 8*i) )&0xff;
+        }
+#ifndef TS_NO_CSA_CK_MSG
+        msg_Dbg( p_caller, "using CSA (de)scrambling with %s key=%x:%x:%x:%x:%x:%x:%x:%x", ((set_odd == 1) ? "odd" : "even" ),
+                 ck[0], ck[1], ck[2], ck[3], ck[4], ck[5], ck[6], ck[7] );
+#endif
+        if ( set_odd == 1 )
+        {
+                 memcpy( c->o_ck, ck, 8 );
+                 csa_ComputeKey( c->o_kk, ck );
+        }
+        else
+        {
+                 memcpy( c->e_ck , ck, 8 );
+                 csa_ComputeKey( c->e_kk , ck );
+        }
+        return VLC_SUCCESS;
+    }
 }
 
 /*****************************************************************************
diff --git a/modules/mux/mpeg/csa.h b/modules/mux/mpeg/csa.h
index 40fc4da..18c1ecc 100644
--- a/modules/mux/mpeg/csa.h
+++ b/modules/mux/mpeg/csa.h
@@ -34,7 +34,7 @@ typedef struct csa_t csa_t;
 csa_t *csa_New( void );
 void   csa_Delete( csa_t * );
 
-void   csa_SetCW( csa_t *, uint8_t o_ck[8], uint8_t e_ck[8] );
+int    csa_SetCW( vlc_object_t *p_caller, csa_t *c, char *psz_ck, int set_odd );
 
 void   csa_Decrypt( csa_t *, uint8_t *pkt, int i_pkt_size );
 void   csa_Encrypt( csa_t *, uint8_t *pkt, int i_pkt_size, int b_odd );
diff --git a/modules/mux/mpeg/ts.c b/modules/mux/mpeg/ts.c
index bd4b70b..f994f60 100644
--- a/modules/mux/mpeg/ts.c
+++ b/modules/mux/mpeg/ts.c
@@ -771,38 +771,20 @@ static int Open( vlc_object_t *p_this )
     var_Get( p_mux, SOUT_CFG_PREFIX "csa-ck", &val );
     if( val.psz_string && *val.psz_string )
     {
-        char *psz = val.psz_string;
+        int i_res;
 
-        /* skip 0x */
-        if( psz[0] == '0' && ( psz[1] == 'x' || psz[1] == 'X' ) )
-        {
-            psz += 2;
-        }
-        if( strlen( psz ) != 16 )
+        p_sys->csa = csa_New();
+
+        i_res = csa_SetCW( (vlc_object_t*)p_mux, p_sys->csa, val.psz_string, 1 );
+        if( i_res != VLC_SUCCESS || csa_SetCW( (vlc_object_t*)p_mux, p_sys->csa, val.psz_string, 0 ) != VLC_SUCCESS )
         {
-            msg_Dbg( p_mux, "invalid csa ck (it must be 16 chars long)" );
+            csa_Delete( p_sys->csa );
         }
-        else
-        {
-            uint64_t i_ck = strtoull( psz, NULL, 16 );
-            uint8_t  ck[8];
-            int      i;
 
-            for( i = 0; i < 8; i++ )
-            {
-                ck[i] = ( i_ck >> ( 56 - 8*i) )&0xff;
-            }
-#ifndef TS_NO_CSA_CK_MSG
-            msg_Dbg( p_mux, "using CSA scrambling with ck=%x:%x:%x:%x:%x:%x:%x:%x",
-                     ck[0], ck[1], ck[2], ck[3], ck[4], ck[5], ck[6], ck[7] );
-#endif
-            p_sys->csa = csa_New();
             if( p_sys->csa )
             {
                 vlc_value_t pkt_val;
 
-                csa_SetCW( p_sys->csa, ck, ck );
-
                 var_Get( p_mux, SOUT_CFG_PREFIX "csa-pkt", &pkt_val );
                 if( pkt_val.i_int < 12 || pkt_val.i_int > 188 )
                 {
@@ -813,7 +795,6 @@ static int Open( vlc_object_t *p_this )
                 else p_sys->i_csa_pkt_size = pkt_val.i_int;
                 msg_Dbg( p_mux, "encrypting %d bytes of packet", p_sys->i_csa_pkt_size );
             }
-        }
     }
     free( val.psz_string );
 
-- 
1.5.5

